from enum import Enum
import shutil
from colorama import Fore, Style

from .print_formatter_abstract import FormatterAbstract


class MessageType(Enum):
    """
    Callback Message Type
    """
    CommRead = (0,)
    CommWrite = (1,)
    System = (2,)
    Log = (3,)
    UserInstruction = (4,)
    ExecutionLog = (5,)
    Error = (6,)


class PrintFormatter(FormatterAbstract):
    """
    Formatted prints class
    """
    def set_cb(self, render_html_cb=None, log_callback=None, is_manuf=False):
        """
        Set callback parameters (constructor)

        Args:
            render_html_cb (Callable[[str], None], optional): Web App output callback object. Defaults to None.
            log_callback (Callable[[str], None], optional): Logger callback object. Defaults to None.
            is_manuf (bool, optional): Server callback flag. Defaults to False.
        """
        self._render_html_cb = render_html_cb
        self._log_callback = log_callback
        self._is_manuf = is_manuf

    def callback(self, message, message_type):
        """
        Callback message wrapper

        Args:
            message (str): output message
            message_type (MessageType): Enum object with message type code
        """
        try:
            if self.is_log_cb():
                self._log_callback(message, message_type)
        except:
            pass

    def is_log_cb(self):
        """
        Check if callback initialized 

        Returns:
            Callable[[str]: Output callback object if initialized.
                            Otherwise, return None.
        """
        return self._log_callback is not None

    def print_dict(self, message):
        """
        Print dictionary content

        Args:
            message (str): output message
        """
        self.callback(message, MessageType.Log.value)
        for i in message:
            print(f'{i}: {message[i]}')

    def print_short(self, message):
        """
        Print unformatted message

        Args:
            message (str): output message
        """
        self.callback(message, MessageType.Log.value)
        print(message)

    def print_message(self, message):
        """
        Print formatted message in green color (MSG)

        Args:
            message (str): output message
        """
        self.callback(message, MessageType.ExecutionLog.value)
        print(Fore.GREEN + "-" * 200)
        print((Fore.GREEN + message).center(shutil.get_terminal_size().columns))
        print(Fore.GREEN + "-" * 200)
        self.reset_color()

    def print_short_blue(self, message):
        """
        Print regular message in blue color

        Args:
            message (str): output message
        """
        self.callback(message, MessageType.Log.value)
        print((Fore.BLUE + message))
        self.reset_color()

    def print_short_green(self, message):
        """
        Print regular message in green color

        Args:
            message (str): output message
        """
        self.callback(message, MessageType.Log.value)
        print((Fore.GREEN + message))
        self.reset_color()

    def print_error_message(self, message):
        """
        Print formatted message in red color (ERR)

        Args:
            message (str): output message
        """
        self.callback(message, MessageType.Error.value)
        print(Fore.RED + "-" * 200)
        print((Fore.RED + message).center(shutil.get_terminal_size().columns))
        print(Fore.RED + "-" * 200)
        self.reset_color()

    @ staticmethod
    def reset_color():
        """
        Reset color formatting to white color - default
        """
        print(Style.RESET_ALL)



from typing import Callable, Any
from abc import ABC, abstractmethod


class FormatterAbstract(ABC):
    @abstractmethod
    def set_cb(self, render_html_cb: Callable[[str], None], log_callback: Callable[[str], None], is_manuf: bool):
        pass

    @abstractmethod
    def callback(self, message: Any, message_type: Any):
        pass

    @abstractmethod
    def is_log_cb(self):
        pass

    @abstractmethod
    def print_dict(self, message: dict):
        pass

    @abstractmethod
    def print_short(self, message: str):
        pass

    @abstractmethod
    def print_message(self, message: str):
        pass

    @abstractmethod
    def print_short_blue(self, message: str):
        pass

    @abstractmethod
    def print_error_message(self, message: str):
        pass



import unittest
from unittest.mock import patch, MagicMock
from opsys_logger.print_formatter import PrintFormatter, MessageType


class Test(unittest.TestCase):
    @ classmethod
    def setUp(self):
        pass

    @ classmethod
    def setUpClass(cls):
        pass

    @ classmethod
    def tearDownClass(cls):
        pass

    @ patch.object(PrintFormatter, 'set_cb')
    def test_set_cb(self, print_formatter_mock: MagicMock):
        print_format = PrintFormatter()
        render_html_cb = None 
        log_callback = None
        is_manuf = False
        print_format.set_cb(render_html_cb, log_callback, is_manuf)
        print_formatter_mock.assert_called_once_with(None, None, False)
        
    @ patch.object(PrintFormatter, 'callback')
    def test_callback(self, print_formatter_mock: MagicMock):
        print_format = PrintFormatter()
        message = 'log message'
        message_type = MessageType.Log.value
        print_format.callback(message, message_type)
        print_formatter_mock.assert_called_once_with('log message', (3,))
        
    @ patch.object(PrintFormatter, 'is_log_cb')
    def test_is_log_cb(self, print_formatter_mock: MagicMock):
        print_format = PrintFormatter()
        print_format.is_log_cb()
        print_formatter_mock.assert_called_once_with()
        
    @ patch.object(PrintFormatter, 'print_dict')
    def test_print_dict(self, print_formatter_mock: MagicMock):
        print_format = PrintFormatter()
        message = "print dict"
        print_format.print_dict(message)
        print_formatter_mock.assert_called_once_with("print dict")
        
    @ patch.object(PrintFormatter, 'print_short')
    def test_print_short(self, print_formatter_mock: MagicMock):
        print_format = PrintFormatter()
        message = "print short"
        print_format.print_short(message)
        print_formatter_mock.assert_called_once_with("print short")
        
    @ patch.object(PrintFormatter, 'print_message')
    def test_print_message(self, print_formatter_mock: MagicMock):
        print_format = PrintFormatter()
        message = "print message"
        print_format.print_message(message)
        print_formatter_mock.assert_called_once_with("print message")
        
    @ patch.object(PrintFormatter, 'print_short_blue')
    def test_print_short_blue(self, print_formatter_mock: MagicMock):
        print_format = PrintFormatter()
        message = "print short blue"
        print_format.print_short_blue(message)
        print_formatter_mock.assert_called_once_with("print short blue")
        
    @ patch.object(PrintFormatter, 'print_short_green')
    def test_print_short_green(self, print_formatter_mock: MagicMock):
        print_format = PrintFormatter()
        message = "print short green"
        print_format.print_short_green(message)
        print_formatter_mock.assert_called_once_with("print short green")
        
    @ patch.object(PrintFormatter, 'print_error_message')
    def test_print_error_message(self, print_formatter_mock: MagicMock):
        print_format = PrintFormatter()
        message = "print error message"
        print_format.print_error_message(message)
        print_formatter_mock.assert_called_once_with("print error message")
        
    @ patch.object(PrintFormatter, 'reset_color')
    def test_reset_color(self, print_formatter_mock: MagicMock):
        print_format = PrintFormatter()
        print_format.reset_color()
        print_formatter_mock.assert_called_once_with()


if __name__ == '__main__':
    unittest.main()
